# Generated by Django 3.2.15 on 2023-01-02 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0044_auto_20220831_1241'),
    ]

    operations = [
        migrations.AddField(
            model_name='sellerpart',
            name='link',
            field=models.URLField(blank=True, null=True),
        ),
    ]
