# Generated by Django 2.2.9 on 2020-02-03 06:18

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0032_auto_20200126_1806'),
    ]

    operations = [
        migrations.AddField(
            model_name='organization',
            name='number_scheme',
            field=models.CharField(choices=[('I', 'Intelligent'), ('S', 'Semi-Intelligent')], default='S', max_length=1),
        ),
        migrations.AlterField(
            model_name='organization',
            name='number_item_len',
            field=models.PositiveIntegerField(default=3, validators=[django.core.validators.MinValueValidator(3), django.core.validators.MaxValueValidator(128)]),
        ),
        migrations.AlterField(
            model_name='part',
            name='number_item',
            field=models.CharField(blank=True, default=None, max_length=128, validators=[django.core.validators.RegexValidator('^[0-9]*$', 'Only numeric characters are allowed.')]),
        ),
    ]
