# Generated by Django 2.2.9 on 2019-12-22 22:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0025_auto_20191221_1907'),
    ]

    operations = [
        migrations.AlterField(
            model_name='partrevision',
            name='searchable_synopsis',
            field=models.CharField(blank=True, db_index=True, default='', editable=False, max_length=255, null=True),
        ),
        migrations.AlterIndexTogether(
            name='part',
            index_together={('organization', 'number_class')},
        ),
    ]
