# Generated by Django 3.2.16 on 2023-01-29 21:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bom', '0045_sellerpart_link'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='sellerpart',
            unique_together=set(),
        ),
    ]
